package mods.immibis.macroblocks;

import java.util.List;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.common.util.ForgeDirection;

/**
 * METADATA ASSIGNMENT:
 * 0 = bricks
 * 1 = carved bricks
 * 2 = vertical pillar
 * 3 = horizontal (X) pillar (never used in item form)
 * 4 = horizontal (Z) pillar (never used in item form)
 * 5 = small bricks
 * 6 = large tiles
 * 7 = small tiles
 * 8 = ornate large tiles
 */
public class BlockMB extends Block {
	
	public static final int META_BRICKS = 0;
	public static final int META_CARVED_BRICKS = 1;
	public static final int META_PILLAR = 2;
	public static final int META_X_PILLAR = 3;
	public static final int META_Z_PILLAR = 4;
	public static final int META_SMALL_BRICKS = 5;
	public static final int META_LARGE_TILES = 6;
	public static final int META_SMALL_TILES = 7;
	public static final int META_ORNATE_LARGE_TILES = 8;
	
	protected String unlocalizedMaterialName;
	protected ResourceLocation baseTextureName;
	
	protected BlockMB(ResourceLocation baseTextureName, String unlocalizedMaterialName) {
		super(Material.rock);
		setCreativeTab(CreativeTabs.tabBlock);
		
		this.unlocalizedMaterialName = unlocalizedMaterialName;
		this.baseTextureName = baseTextureName;
		
		setBlockName("immibis_macroblocks."+unlocalizedMaterialName);
	}
	
	private IIcon iBricks, iCarvedBricks, iPillarVert, iPillarHorz, iPillarEnd, iSmallBricks;
	private IIcon iLargeTiles, iSmallTiles, iOrnateLargeTiles;
	
	@SideOnly(Side.CLIENT)
	@Override
	public void registerBlockIcons(IIconRegister reg) {
		iBricks = createIcon(Effect.BRICKS, reg);
		iCarvedBricks = createIcon(Effect.CARVED_BRICKS, reg);
		iPillarVert = createIcon(Effect.PILLAR, reg);
		iPillarHorz = createIcon(Effect.PILLAR_HORIZONTAL, reg);
		iPillarEnd = createIcon(Effect.PILLAR_END, reg);
		iSmallBricks = createIcon(Effect.SMALL_BRICKS, reg);
		iLargeTiles = createIcon(Effect.LARGE_TILES, reg);
		iSmallTiles = createIcon(Effect.SMALL_TILES, reg);
		iOrnateLargeTiles = createIcon(Effect.ORNATE_LARGE_TILES, reg);
	}
	
	@SideOnly(Side.CLIENT)
	protected IIcon createIcon(Effect effect, IIconRegister reg) {
		String iconName = "moartex:gen-"+unlocalizedMaterialName+"-"+effect.name();
		if(!(reg instanceof TextureMap))
			return null;
		
		((TextureMap)reg).setTextureEntry(iconName, new TextureModified(iconName, baseTextureName, 1, 1, 0, 0, effect));
		
		return reg.registerIcon(iconName);
	}

	@Override
	public IIcon getIcon(int side, int meta) {
		switch(meta) {
		case 0: return iBricks;
		case 1: return iCarvedBricks;
		case 2:
			if(side >= 2)
				return iPillarVert;
			else
				return iPillarEnd;
		case 3:
			if(side < 4)
				return iPillarHorz;
			else
				return iPillarEnd;
		case 4:
			if(side < 2)
				return iPillarVert;
			else if(side > 3)
				return iPillarHorz;
			else
				return iPillarEnd;
		case 5:
			return iSmallBricks;
		case 6:
			return iLargeTiles;
		case 7:
			return iSmallTiles;
		case 8:
			return iOrnateLargeTiles;
		}
		return Blocks.stone.getIcon(0, 0);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public void getSubBlocks(Item p_149666_1_, CreativeTabs p_149666_2_, List p_149666_3_) {
		p_149666_3_.add(new ItemStack(this, 1, 0)); // bricks
		p_149666_3_.add(new ItemStack(this, 1, 1)); // carved bricks
		p_149666_3_.add(new ItemStack(this, 1, 2)); // pillar
		p_149666_3_.add(new ItemStack(this, 1, 5)); // small bricks
		p_149666_3_.add(new ItemStack(this, 1, 6)); // large tiles
		p_149666_3_.add(new ItemStack(this, 1, 7)); // small tiles
		p_149666_3_.add(new ItemStack(this, 1, 8)); // ornate large tiles
	}
	
	@Override
	public int damageDropped(int p_149692_1_) {
		if(p_149692_1_ == 3 || p_149692_1_ == 4)
			return 2;
		return p_149692_1_;
	}
	
	private Block base;
	private int baseMeta;
	
	@Override
	public float getPlayerRelativeBlockHardness(EntityPlayer a, World b, int c, int d, int e) {
		if(base != null)
			return base.getPlayerRelativeBlockHardness(a, b, c, d, e);
		return super.getPlayerRelativeBlockHardness(a, b, c, d, e);
	}
	
	@Override
	public float getExplosionResistance(Entity p_149638_1_) {
		if(base != null)
			return base.getExplosionResistance(p_149638_1_);
		return super.getExplosionResistance(p_149638_1_);
	}
	
	@Override
	public boolean canCreatureSpawn(EnumCreatureType type, IBlockAccess world, int x, int y, int z) {
		if(base != null)
			return base.canCreatureSpawn(type, world, x, y, z);
		return super.canCreatureSpawn(type, world, x, y, z);
	}
	
	@Override
	public boolean canBeReplacedByLeaves(IBlockAccess world, int x, int y, int z) {
		if(base != null)
			return base.canBeReplacedByLeaves(world, x, y, z);
		return super.canBeReplacedByLeaves(world, x, y, z);
	}
	
	@Override
	public boolean canSustainLeaves(IBlockAccess world, int x, int y, int z) {
		if(base != null)
			return base.canSustainLeaves(world, x, y, z);
		return super.canSustainLeaves(world, x, y, z);
	}
	
	@Override
	public boolean canSustainPlant(IBlockAccess world, int x, int y, int z, ForgeDirection direction, IPlantable plantable) {
		if(base != null)
			return base.canSustainLeaves(world, x, y, z);
		return super.canSustainPlant(world, x, y, z, direction, plantable);
	}
	
	@Override
	public int getFireSpreadSpeed(IBlockAccess world, int x, int y, int z, ForgeDirection face) {
		if(base != null)
			return base.getFireSpreadSpeed(world, x, y, z, face);
		return super.getFireSpreadSpeed(world, x, y, z, face);
	}
	
	@Override
	public int getFlammability(IBlockAccess world, int x, int y, int z, ForgeDirection face) {
		if(base != null)
			return base.getFlammability(world, x, y, z, face);
		return super.getFlammability(world, x, y, z, face);
	}
	
	@Override
	public float getExplosionResistance(Entity par1Entity, World world, int x, int y, int z, double explosionX, double explosionY, double explosionZ) {
		if(base != null)
			return base.getExplosionResistance(par1Entity, world, x, y, z, explosionX, explosionY, explosionZ);
		return super.getExplosionResistance(par1Entity, world, x, y, z, explosionX, explosionY, explosionZ);
	}
	
	@Override
	public int getMobilityFlag() {
		if(base != null)
			return base.getMobilityFlag();
		return super.getMobilityFlag();
	}
	
	@Override
	public int getLightOpacity() {
		if(base != null)
			return base.getLightOpacity();
		return super.getLightOpacity();
	}
	
	@Override
	public int getLightValue() {
		if(base != null)
			return base.getLightValue();
		return super.getLightValue();
	}
	
	@Override
	public int getHarvestLevel(int metadata) {
		if(base != null)
			return base.getHarvestLevel(baseMeta);
		return super.getHarvestLevel(metadata);
	}
	
	@Override
	public String getHarvestTool(int metadata) {
		if(base != null)
			return base.getHarvestTool(baseMeta);
		return super.getHarvestTool(metadata);
	}
	
	@Override
	public int isProvidingStrongPower(IBlockAccess w, int x, int y, int z, int dir) {
		return base == null ? 0 : base.isProvidingStrongPower(w, x, y, z, dir);
	}
	
	@Override
	public int isProvidingWeakPower(IBlockAccess w, int x, int y, int z, int dir) {
		return base == null ? 0 : base.isProvidingWeakPower(w, x, y, z, dir);
	}
	
	@Override
	public boolean isBurning(IBlockAccess world, int x, int y, int z) {
		return base != null && base.isBurning(world, x, y, z);
	}
	
	@Override
	public boolean isBeaconBase(IBlockAccess world, int x, int y, int z, int bx, int by, int bz) {
		return base != null && base.isBeaconBase(world, x, y, z, bx, by, bz);
	}
	
	@Override
	public boolean isToolEffective(String type, int metadata) {
		return base == null || base.isToolEffective(type, metadata);
	}
	
	@Override
	public boolean isFertile(World world, int x, int y, int z) {
		return base == null || base.isFertile(world, x, y, z);
	}
	
	@Override
	public boolean isFireSource(World world, int x, int y, int z, ForgeDirection side) {
		return base != null && base.isFireSource(world, x, y, z, side);
	}
	
	

	public void copyPropertiesFrom(Block base, int baseMeta) {
		this.base = base;
		this.baseMeta = baseMeta;
	}
}
