package mods.immibis.macroblocks;

import java.util.List;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.util.ResourceLocation;

/**
 * METADATA ASSIGNMENT:
 * 0 = UNUSED
 * 1 = UNUSED
 * 2 = vertical pillar
 * 3 = horizontal (X) pillar (never used in item form)
 * 4 = horizontal (Z) pillar (never used in item form)
 * 5 = UNUSED
 * 6 = large tiles
 * 7 = small tiles
 * 8 = ornate large tiles
 */
public class BlockMBRailcraftBricks extends BlockMB {
	
	protected BlockMBRailcraftBricks(ResourceLocation baseTextureName, String unlocalizedMaterialName) {
		super(baseTextureName, unlocalizedMaterialName);
	}
	
	private IIcon iPillarVert, iPillarHorz, iPillarEnd;
	private IIcon iLargeTiles, iSmallTiles, iOrnateLargeTiles;
	
	@SideOnly(Side.CLIENT)
	@Override
	public void registerBlockIcons(IIconRegister reg) {
		iPillarVert = createIcon(Effect.PILLAR, reg);
		iPillarHorz = createIcon(Effect.PILLAR_HORIZONTAL, reg);
		iPillarEnd = createIcon(Effect.PILLAR_END, reg);
		iLargeTiles = createIcon(Effect.LARGE_TILES, reg);
		iSmallTiles = createIcon(Effect.SMALL_TILES, reg);
		iOrnateLargeTiles = createIcon(Effect.ORNATE_LARGE_TILES, reg);
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	protected IIcon createIcon(Effect effect, IIconRegister reg) {
		String iconName = "moartex:gen-"+unlocalizedMaterialName+"-"+effect.name();
		if(!(reg instanceof TextureMap))
			return null;
		
		((TextureMap)reg).setTextureEntry(iconName, new TextureModified(iconName, baseTextureName, 6, 1, 2, 0, effect));
		
		return reg.registerIcon(iconName);
	}

	@Override
	public IIcon getIcon(int side, int meta) {
		switch(meta) {
		case 2:
			if(side >= 2)
				return iPillarVert;
			else
				return iPillarEnd;
		case 3:
			if(side < 4)
				return iPillarHorz;
			else
				return iPillarEnd;
		case 4:
			if(side < 2)
				return iPillarVert;
			else if(side > 3)
				return iPillarHorz;
			else
				return iPillarEnd;
		case 6:
			return iLargeTiles;
		case 7:
			return iSmallTiles;
		case 8:
			return iOrnateLargeTiles;
		}
		return Blocks.stone.getIcon(0, 0);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public void getSubBlocks(Item p_149666_1_, CreativeTabs p_149666_2_, List p_149666_3_) {
		p_149666_3_.add(new ItemStack(this, 1, 2)); // pillar
		p_149666_3_.add(new ItemStack(this, 1, 6)); // large tiles
		p_149666_3_.add(new ItemStack(this, 1, 7)); // small tiles
		p_149666_3_.add(new ItemStack(this, 1, 8)); // ornate large tiles
	}
}
