package mods.immibis.macroblocks;

import java.util.List;

import mods.immibis.macroblocks.MacroblocksMod.MaterialInfo;
import mods.immibis.macroblocks.MacroblocksMod.MaterialInfoNormal;
import mods.railcraft.common.blocks.aesthetics.brick.BrickVariant;
import mods.railcraft.common.blocks.aesthetics.brick.EnumBrick;
import mods.railcraft.common.blocks.aesthetics.cube.BlockCube;
import mods.railcraft.common.blocks.aesthetics.cube.EnumCube;
import net.minecraft.block.Block;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import cpw.mods.fml.common.registry.GameRegistry;

class Loader_Railcraft {
	static class MaterialInfoRailcraftBrick extends MacroblocksMod.MaterialInfo {
		public final String side_texture_name;
		public final String unlocalized_material_name;
		private final EnumBrick rc_brick_type;
		public BlockMB mt_block;
		public MaterialInfoRailcraftBrick(String stn, String umn, EnumBrick rc_brick_type) {
			super("railcraft_"+umn+"_block");
			
			this.side_texture_name = stn;
			this.unlocalized_material_name = umn;
			this.rc_brick_type = rc_brick_type;
		}
		
		@Override
		void init() {
			mt_block = new BlockMBRailcraftBricks(new ResourceLocation("railcraft", "textures/blocks/"+side_texture_name+".png"), unlocalized_material_name);
			GameRegistry.registerBlock(mt_block, ItemBlockMB.class, this.blockID);
		}
		
		@Override
		void postinit() {
			ItemStack bricks = rc_brick_type.get(BrickVariant.FITTED);
			ItemStack small_bricks = rc_brick_type.get(BrickVariant.BRICK);
			ItemStack carved_bricks = rc_brick_type.get(BrickVariant.ORNATE);
			ItemStack pillar = new ItemStack(mt_block, 1, BlockMB.META_PILLAR);
			ItemStack large_tiles = new ItemStack(mt_block, 1, BlockMB.META_LARGE_TILES);
			ItemStack small_tiles = new ItemStack(mt_block, 1, BlockMB.META_SMALL_TILES);
			ItemStack ornate_large_tiles = new ItemStack(mt_block, 1, BlockMB.META_ORNATE_LARGE_TILES);
			
			// bricks -> small bricks (2x2)
			// (not added; Railcraft-internal recipe)
			
			// bricks -> carved bricks (3x3 with hole)
			// (not added; Railcraft-internal recipe)
			
			// bricks -> pillar (1x2)
			GameRegistry.addRecipe(MacroblocksMod.copyWithSize(pillar, 2), "X", "X", 'X', bricks);
			
			// bricks -> large tiles (diamond shape)
			GameRegistry.addRecipe(MacroblocksMod.copyWithSize(large_tiles, 4), " X ", "X X", " X ", 'X', bricks);
			
			// small bricks -> small tiles (diamond shape)
			GameRegistry.addRecipe(MacroblocksMod.copyWithSize(small_tiles, 4), " X ", "X X", " X ", 'X', small_bricks);
			
			// large tiles -> ornate large tiles (2x2)
			GameRegistry.addRecipe(MacroblocksMod.copyWithSize(ornate_large_tiles, 4), "XX", "XX", 'X', large_tiles);
			
			
			
			// reverse recipes
			// bricks -> base
			// (not added; Railcraft-internal recipe)
			// small bricks -> base
			// (not added; Railcraft-internal recipe)
			// carved bricks -> bricks
			// (not added; Railcraft-internal recipe)
			// pillar -> bricks
			GameRegistry.addRecipe(bricks, "X", 'X', pillar);
			// large tiles -> bricks
			GameRegistry.addRecipe(bricks, "X", 'X', large_tiles);
			// small tiles -> small bricks
			GameRegistry.addRecipe(small_bricks, "X", 'X', small_tiles);
			// ornate large tiles -> large tiles
			GameRegistry.addRecipe(large_tiles, "X", 'X', ornate_large_tiles);
			
			
			
			
			
			// TODO test block properties
			
			Block base = rc_brick_type.getBlock();
			if(base != null) {
				mt_block.copyPropertiesFrom(base, bricks.getItemDamage());
			}			
		}
	}
	
	public Loader_Railcraft(List<MacroblocksMod.MaterialInfo> materials) {
		materials.add(new MaterialInfoRailcraftBrick("brick.abyssal", "railcraft_abyssal", EnumBrick.ABYSSAL));
		materials.add(new MaterialInfoRailcraftBrick("brick.bleachedbone", "railcraft_bleachedbone", EnumBrick.BLEACHEDBONE));
		materials.add(new MaterialInfoRailcraftBrick("brick.bloodstained", "railcraft_bloodstained", EnumBrick.BLOODSTAINED));
		materials.add(new MaterialInfoRailcraftBrick("brick.frostbound", "railcraft_frostbound", EnumBrick.FROSTBOUND));
		materials.add(new MaterialInfoRailcraftBrick("brick.infernal", "railcraft_infernal", EnumBrick.INFERNAL));
		materials.add(new MaterialInfoRailcraftBrick("brick.quarried", "railcraft_quarried", EnumBrick.QUARRIED));
		materials.add(new MaterialInfoRailcraftBrick("brick.sandy", "railcraft_sandy", EnumBrick.SANDY));
		materials.add(new MaterialInfoNormal("railcraft:cube.coke", "railcraft_coke", BlockCube.getBlock(), EnumCube.COKE_BLOCK.ordinal()));
		materials.add(new MaterialInfoNormal("railcraft:concrete", "railcraft_concrete", BlockCube.getBlock(), EnumCube.CONCRETE_BLOCK.ordinal()));
		materials.add(new MaterialInfoNormal("railcraft:cube.steel", "railcraft_steel", BlockCube.getBlock(), EnumCube.STEEL_BLOCK.ordinal()));
	}
}
