package mods.immibis.macroblocks;

import java.util.ArrayList;
import java.util.List;

import mods.immibis.core.api.FMLModInfo;
import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLMissingMappingsEvent;
import cpw.mods.fml.common.event.FMLMissingMappingsEvent.MissingMapping;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.registry.GameData;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.common.registry.GameRegistry.Type;
import cpw.mods.fml.common.registry.RegistryDelegate;

@Mod(modid="ImmibisMacroblocks", name="Immibis's Macroblocks", version="59.1.1")
@FMLModInfo(authors="immibis", description="Adds some procedurally-generated brick types", url="")
public class MacroblocksMod {
	
	static abstract class MaterialInfo {
		final String blockID;
		public MaterialInfo(String blockID) {
			this.blockID = blockID;
		}
		abstract void init();
		abstract void postinit();
	}
	
	static class MaterialInfoNormal extends MaterialInfo {
		public final String side_texture_name;
		public final String unlocalized_material_name;
		private final RegistryDelegate<Block> base_block;
		private final int baseBlockMeta;
		public BlockMB mt_block;
		public MaterialInfoNormal(String stn, String umn, Block baseBlock, int baseMeta) {
			super(umn+"_block");
			
			this.side_texture_name = stn;
			this.unlocalized_material_name = umn;
			this.base_block = baseBlock.delegate;
			this.baseBlockMeta = baseMeta;
		}
		public MaterialInfoNormal(String stn, String umn, Block baseBlock) {
			this(stn, umn, baseBlock, 0);
		}
		public ItemStack getBaseStack() {
			Block b = base_block.get();
			if(b == null)
				return null;
			return new ItemStack(b, 1, baseBlockMeta);
		}
		public Block getBaseBlock() {
			return base_block.get();
		}
		
		@Override
		void init() {
			ResourceLocation baseResLoc;
			baseResLoc = new ResourceLocation(side_texture_name);
			baseResLoc = new ResourceLocation(baseResLoc.getResourceDomain(), "textures/blocks/"+baseResLoc.getResourcePath()+".png");
			mt_block = new BlockMB(baseResLoc, unlocalized_material_name);
			GameRegistry.registerBlock(mt_block, ItemBlockMB.class, this.blockID);
		}
		
		@Override
		void postinit() {

			ItemStack baseStack = getBaseStack();
			
			ItemStack bricks = new ItemStack(mt_block, 1, BlockMB.META_BRICKS);
			ItemStack small_bricks = new ItemStack(mt_block, 1, BlockMB.META_SMALL_BRICKS);
			ItemStack carved_bricks = new ItemStack(mt_block, 1, BlockMB.META_CARVED_BRICKS);
			ItemStack pillar = new ItemStack(mt_block, 1, BlockMB.META_PILLAR);
			ItemStack large_tiles = new ItemStack(mt_block, 1, BlockMB.META_LARGE_TILES);
			ItemStack small_tiles = new ItemStack(mt_block, 1, BlockMB.META_SMALL_TILES);
			ItemStack ornate_large_tiles = new ItemStack(mt_block, 1, BlockMB.META_ORNATE_LARGE_TILES);
			
			if(baseStack != null) {
				
				// base -> bricks
				GameRegistry.addRecipe(copyWithSize(bricks, 4), "XX", "XX", 'X', baseStack); 
				// alternate in case of recipe conflicts
				GameRegistry.addRecipe(copyWithSize(bricks, 8), "XXX", "XYX", "XXX", 'X', baseStack, 'Y', Blocks.stonebrick);
			}
			
			// bricks -> small bricks (2x2)
			GameRegistry.addRecipe(copyWithSize(small_bricks, 4), "XX", "XX", 'X', bricks);
			
			// bricks -> carved bricks (3x3 with hole)
			GameRegistry.addRecipe(copyWithSize(carved_bricks, 8), "XXX", "X X", "XXX", 'X', bricks);
			
			// bricks -> pillar (1x2)
			GameRegistry.addRecipe(copyWithSize(pillar, 2), "X", "X", 'X', bricks);
			
			// bricks -> large tiles (diamond shape)
			GameRegistry.addRecipe(copyWithSize(large_tiles, 4), " X ", "X X", " X ", 'X', bricks);
			
			// small bricks -> small tiles (diamond shape)
			GameRegistry.addRecipe(copyWithSize(small_tiles, 4), " X ", "X X", " X ", 'X', small_bricks);
			
			// large tiles -> ornate large tiles (2x2)
			GameRegistry.addRecipe(copyWithSize(ornate_large_tiles, 4), "XX", "XX", 'X', large_tiles);
			
			
			
			// reverse recipes
			// bricks -> base
			GameRegistry.addRecipe(baseStack, "X", 'X', bricks);
			// small bricks -> base
			GameRegistry.addRecipe(baseStack, "X", 'X', small_bricks);
			// carved bricks -> bricks
			GameRegistry.addRecipe(bricks, "X", 'X', carved_bricks);
			// pillar -> bricks
			GameRegistry.addRecipe(bricks, "X", 'X', pillar);
			// large tiles -> bricks
			GameRegistry.addRecipe(bricks, "X", 'X', large_tiles);
			// small tiles -> small bricks
			GameRegistry.addRecipe(small_bricks, "X", 'X', small_tiles);
			// ornate large tiles -> large tiles
			GameRegistry.addRecipe(large_tiles, "X", 'X', ornate_large_tiles);
			
			
			
			
			
			// TODO test block properties
			
			Block base = getBaseBlock();
			if(base != null) {
				mt_block.copyPropertiesFrom(base, baseBlockMeta);
			}			
		}
	}
	

	
	
	private static final List<MaterialInfo> materials = new ArrayList<>();
	
	
	
	@EventHandler
	public void init(FMLInitializationEvent evt) {
		materials.add(new MaterialInfoNormal("end_stone", "endstone", Blocks.end_stone));
		materials.add(new MaterialInfoNormal("netherrack", "netherrack", Blocks.netherrack));
		materials.add(new MaterialInfoNormal("lapis_block", "lapis", Blocks.lapis_block));
		materials.add(new MaterialInfoNormal("redstone_block", "redstone", Blocks.redstone_block));
		materials.add(new MaterialInfoNormal("iron_block", "iron", Blocks.iron_block));
		materials.add(new MaterialInfoNormal("gold_block", "gold", Blocks.gold_block));
		materials.add(new MaterialInfoNormal("diamond_block", "diamond", Blocks.diamond_block));
		materials.add(new MaterialInfoNormal("coal_block", "coal", Blocks.coal_block));
		materials.add(new MaterialInfoNormal("cobblestone", "cobblestone", Blocks.cobblestone));
		materials.add(new MaterialInfoNormal("sandstone_bottom", "sandstone", Blocks.sandstone));
		materials.add(new MaterialInfoNormal("obsidian", "obsidian", Blocks.obsidian));
		materials.add(new MaterialInfoNormal("soul_sand", "soul_sand", Blocks.soul_sand));
		materials.add(new MaterialInfoNormal("glowstone", "glowstone", Blocks.glowstone));
		
		materials.add(new MaterialInfoNormal("snow", "snow", Blocks.snow));
		materials.add(new MaterialInfoNormal("clay", "clay", Blocks.clay));
		materials.add(new MaterialInfoNormal("sponge", "sponge", Blocks.sponge));
		materials.add(new MaterialInfoNormal("bedrock", "bedrock", Blocks.bedrock));
		materials.add(new MaterialInfoNormal("wool_colored_white", "wool", Blocks.wool));
		materials.add(new MaterialInfoNormal("cobblestone_mossy", "moss_stone", Blocks.mossy_cobblestone));
		materials.add(new MaterialInfoNormal("pumpkin_side", "pumpkin", Blocks.pumpkin));
		materials.add(new MaterialInfoNormal("melon_side", "melon", Blocks.melon_block));
		materials.add(new MaterialInfoNormal("emerald_block", "emerald", Blocks.emerald_block));
		materials.add(new MaterialInfoNormal("nether_brick", "netherbrick", Blocks.nether_brick));
		materials.add(new MaterialInfoNormal("ice_packed", "packed_ice", Blocks.packed_ice));
		
		
		try {
			
			if(Loader.isModLoaded("Railcraft")) {
				String packageName = MacroblocksMod.class.getName();
				packageName = packageName.substring(0, packageName.lastIndexOf('.'));
				Class.forName(packageName+".Loader_Railcraft").getConstructor(List.class).newInstance(materials);
			}
		
		} catch(ReflectiveOperationException e) {
			throw new RuntimeException(e);
		}
		
		for(MaterialInfo m : materials) {
			m.init();
		}
	}
	
	static ItemStack copyWithSize(ItemStack s, int size) {
		s = s.copy();
		s.stackSize = size;
		return s;
	}
	
	@EventHandler
	public void postinit(FMLPostInitializationEvent evt) {
		for(MaterialInfo m : materials) {
			m.postinit();
		}
	}
	
	@EventHandler
	public void renameModMappings(FMLMissingMappingsEvent e) {
		String oldModID = "MoarTex";
		String newModID = MacroblocksMod.class.getAnnotation(Mod.class).modid();
		for(MissingMapping m : e.getAll()) {
			if(m.name.startsWith(oldModID+":")) {
				String newName = newModID+m.name.substring(oldModID.length());
				if(m.type == Type.BLOCK)
					m.remap(GameData.getBlockRegistry().getObject(newName));
				else
					m.remap(GameData.getItemRegistry().getObject(newName));
			}
		}
	}
}
