package mods.immibis.macroblocks;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.List;

import javax.imageio.ImageIO;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.resources.IResource;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.client.resources.data.AnimationMetadataSection;
import net.minecraft.util.ResourceLocation;

@SideOnly(Side.CLIENT)
public class TextureModified extends TextureAtlasSprite {

	private final Effect effect;
	private final ResourceLocation baseResLoc;
	
	private int baseCX, baseCY, baseX, baseY;
	
	protected TextureModified(String name, ResourceLocation baseResLoc, int baseCX, int baseCY, int baseX, int baseY, Effect effect) {
		super(name);
		this.baseCX = baseCX; this.baseCY = baseCY; this.baseX = baseX; this.baseY = baseY;
		this.baseResLoc = baseResLoc;
		this.effect = effect;
	}
	
	@SideOnly(Side.CLIENT)
	@Override
	public boolean hasCustomLoader(IResourceManager manager, ResourceLocation location) {
		return true;
	}
	
	@SuppressWarnings("unchecked")
	@SideOnly(Side.CLIENT)
	@Override
	public boolean load(IResourceManager manager, ResourceLocation location) {
		
		try {
			IResource iresource = manager.getResource(baseResLoc);
            BufferedImage[] abufferedimage = new BufferedImage[1 + Minecraft.getMinecraft().gameSettings.mipmapLevels];
            abufferedimage[0] = ImageIO.read(iresource.getInputStream());
            
            if(baseCX != 1 || baseCY != 1) {
            	BufferedImage i = abufferedimage[0];
            	int spriteW = i.getWidth()/baseCX;
            	int spriteH = i.getHeight()/baseCY;
            	i = i.getSubimage(spriteW*baseX, spriteH*baseY, spriteW, spriteH);
            	abufferedimage[0] = i;
            }
            
            AnimationMetadataSection animationmetadatasection = (AnimationMetadataSection)iresource.getMetadata("animation");
            loadSprite(abufferedimage, animationmetadatasection, Minecraft.getMinecraft().gameSettings.anisotropicFiltering > 1);
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
		
		for(int[][] frameMipmaps : (List<int[][]>)this.framesTextureData) {
			int[] frameBaseData = frameMipmaps[0];
			for(int k = 1; k < frameMipmaps.length; k++)
				frameMipmaps[k] = null;
				
			BufferedImage tempImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
				
			for(int x = 0; x < width; x++)
				for(int y = 0; y < height; y++)
					tempImage.setRGB(x, y, frameBaseData[x + y*width]);
				
			tempImage = TextureProcessor.doEffect(tempImage, effect);
			
			for(int x = 0; x < width; x++)
				for(int y = 0; y < height; y++)
					frameBaseData[x + y*width] = tempImage.getRGB(x, y);
		}
		
		return false;
	}

}
